"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var ip = require("ip");
var url = require("url");
var requireModule_1 = require("../requireModule");
var pkg = requireModule_1.default('./package.json');
var createPlugins = function (builder, spin) {
    var stack = builder.stack;
    var webpack = requireModule_1.default('webpack');
    var buildNodeEnv = spin.dev ? (stack.hasAny('test') ? 'test' : 'development') : 'production';
    var plugins = [];
    if (spin.dev) {
        plugins.push(new webpack.NamedModulesPlugin());
    }
    else {
        plugins.push(new webpack.optimize.UglifyJsPlugin({ minimize: true }));
        plugins.push(new webpack.LoaderOptionsPlugin({ minimize: true }));
        plugins.push(new webpack.optimize.ModuleConcatenationPlugin());
    }
    var backendUrl = spin.options.backendUrl.replace('{ip}', spin.dev ? ip.address() : 'localhost');
    if (stack.hasAny('dll')) {
        var name = "vendor_" + builder.parent.name;
        plugins = [
            new webpack.DefinePlugin({
                __DEV__: spin.dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
            }),
            new webpack.DllPlugin({
                name: name,
                path: path.join(spin.options.dllBuildDir, name + "_dll.json"),
            }),
        ];
    }
    else {
        if (stack.hasAny('server')) {
            plugins = plugins.concat([
                new webpack.BannerPlugin({
                    banner: 'require("source-map-support").install();',
                    raw: true, entryOnly: false,
                }),
                new webpack.DefinePlugin({
                    __CLIENT__: false, __SERVER__: true, __SSR__: spin.options.ssr && !spin.test,
                    __DEV__: spin.dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
                    __BACKEND_URL__: "\"" + backendUrl + "\"",
                }),
            ]);
        }
        else {
            plugins = plugins.concat([
                new webpack.DefinePlugin({
                    __CLIENT__: true, __SERVER__: false, __SSR__: spin.options.ssr && !spin.test,
                    __DEV__: spin.dev, 'process.env.NODE_ENV': "\"" + buildNodeEnv + "\"",
                    __BACKEND_URL__: (stack.platform !== 'web' ||
                        url.parse(backendUrl).hostname !== 'localhost') ? "\"" + backendUrl + "\"" : false,
                }),
            ]);
            if (stack.hasAny('web')) {
                var ManifestPlugin = requireModule_1.default('webpack-manifest-plugin');
                plugins.push(new ManifestPlugin({
                    fileName: 'assets.json',
                }));
                var hasServer = false;
                for (var name in spin.builders) {
                    if (spin.builders[name].stack.hasAny('server')) {
                        hasServer = true;
                        break;
                    }
                }
                if (!hasServer) {
                    var HtmlWebpackPlugin = requireModule_1.default('html-webpack-plugin');
                    plugins.push(new HtmlWebpackPlugin({
                        template: path.resolve('html-plugin-template.ejs'),
                        inject: 'body',
                    }));
                }
                if (!spin.dev) {
                    plugins.push(new webpack.optimize.CommonsChunkPlugin({
                        name: 'vendor',
                        filename: '[name].[hash].js',
                        minChunks: function (module) {
                            return module.resource && module.resource.indexOf(path.resolve('./node_modules')) === 0;
                        },
                    }));
                }
            }
        }
    }
    return plugins;
};
var getDepsForNode = function (builder, depPlatforms) {
    var deps = [];
    for (var _i = 0, _a = Object.keys(pkg.dependencies); _i < _a.length; _i++) {
        var key = _a[_i];
        var val = depPlatforms[key];
        if (!val || (val.constructor === Array && val.indexOf(builder.parent.name) >= 0) || val === builder.parent.name) {
            deps.push(key);
        }
    }
    return deps;
};
var curWebpackDevPort = 3000;
var webpackPortMap = {};
var createConfig = function (builder, spin) {
    var stack = builder.stack;
    var baseConfig = {
        name: builder.name,
        devtool: spin.dev ? '#cheap-module-source-map' : '#source-map',
        module: {
            rules: [],
        },
        resolve: {
            modules: [path.join(process.cwd(), 'node_modules'), 'node_modules'],
        },
        watchOptions: {
            ignored: /build/,
        },
        bail: !spin.dev,
    };
    var baseDevServerConfig = {
        hot: true,
        contentBase: '/',
        publicPath: '/',
        headers: { 'Access-Control-Allow-Origin': '*' },
        quiet: false,
        noInfo: true,
        stats: { colors: true, chunkModules: false },
    };
    var plugins = createPlugins(builder, spin);
    var config = __assign({}, baseConfig, { plugins: plugins });
    if (stack.hasAny('server')) {
        config = __assign({}, config, { target: 'node', node: {
                __dirname: true,
                __filename: true,
            }, externals: [requireModule_1.default('webpack-node-externals')({
                    whitelist: [/(^webpack|^react-native)/]
                })] });
    }
    if (stack.hasAny('dll')) {
        var name = "vendor_" + builder.parent.name;
        config = __assign({}, config, { devtool: '#cheap-module-source-map', entry: {
                vendor: getDepsForNode(builder, spin.depPlatforms),
            }, output: {
                filename: name + ".[hash]_dll.js",
                path: path.resolve(spin.options.dllBuildDir),
                library: name,
            } });
    }
    else {
        if (stack.hasAny('server')) {
            config = __assign({}, config, { entry: {
                    index: [
                        './src/server/index.js',
                    ],
                }, output: {
                    devtoolModuleFilenameTemplate: spin.dev ? '../../[resource-path]' : undefined,
                    devtoolFallbackModuleFilenameTemplate: spin.dev ? '../../[resource-path];[hash]' : undefined,
                    filename: '[name].js',
                    sourceMapFilename: '[name].[chunkhash].js.map',
                    path: path.resolve(spin.options.backendBuildDir),
                    publicPath: '/',
                } });
        }
        else if (stack.hasAny('web')) {
            var backendUrl = spin.options.backendUrl.replace('{ip}', ip.address());
            var _a = url.parse(backendUrl), protocol = _a.protocol, host = _a.host;
            var backendBaseUrl = protocol + '//' + host;
            var webpackDevPort = void 0;
            if (!builder.webpackDevPort) {
                if (!webpackPortMap[builder.name]) {
                    webpackPortMap[builder.name] = curWebpackDevPort++;
                }
                webpackDevPort = webpackPortMap[builder.name];
            }
            else {
                webpackDevPort = builder.webpackDevPort;
            }
            config = __assign({}, config, { entry: {
                    index: [
                        './src/client/index.js',
                    ],
                }, output: {
                    filename: '[name].[hash].js',
                    path: path.resolve(path.join(spin.options.frontendBuildDir, 'web')),
                    publicPath: '/',
                }, devServer: __assign({}, baseDevServerConfig, { port: webpackDevPort, proxy: {
                        '!/*.hot-update.{json,js}': {
                            target: backendBaseUrl,
                            logLevel: 'info',
                        },
                    } }) });
        }
        else if (stack.hasAny('react-native')) {
            config = __assign({}, config, { entry: {
                    index: [
                        './src/mobile/index.js',
                    ],
                }, output: {
                    filename: "index.mobile.bundle",
                    publicPath: '/',
                    path: path.resolve(path.join(spin.options.frontendBuildDir, builder.name)),
                }, devServer: __assign({}, baseDevServerConfig, { hot: false, port: stack.hasAny('android') ? 3010 : 3020 }) });
        }
        else {
            throw new Error("Unknown platform target: " + stack.platform);
        }
    }
    return config;
};
var WebpackPlugin = (function () {
    function WebpackPlugin() {
    }
    WebpackPlugin.prototype.configure = function (builder, spin) {
        var stack = builder.stack;
        if (stack.hasAny('webpack')) {
            builder.config = builder.config || {};
            builder.config = spin.merge(builder.config, createConfig(builder, spin));
        }
    };
    return WebpackPlugin;
}());
exports.default = WebpackPlugin;
//# sourceMappingURL=WebpackPlugin.js.map